#include "cppdefs.h"
      SUBROUTINE get_inwave_data (ng)
#ifdef INWAVE_MODEL
!
!svn $Id: get_inwave_data.F 790 2008-10-14 19:32:07Z jcwarner $
!================================================== John C. Warner =====
!                                                                      !
!  This routine reads in forcing data   .............................. !
!  from input NetCDF files.  If there is more than one time-record,    !
!  data  is loaded into global two-time record arrays. The actual      !
!  interpolation is carried elsewhere.                                 !
!                                                                      !
!  Currently,  this routine is only executed in serial mode by the     !
!  main thread.                                                        !
!                                                                      !
!=======================================================================
!
      USE mod_param
      USE mod_forces
      USE mod_grid
      USE mod_iounits
      USE mod_ncparam
      USE mod_scalars
      USE mod_stepping
      USE mod_inwave_bound
      USE mod_inwave_params
      USE mod_inwave_vars
!
      implicit none
!
!  Imported variable declarations.
!
      integer, intent(in) :: ng
!
!  Local variable declarations.
!
      logical, dimension(3) :: update =                                 &
     &         (/ .FALSE., .FALSE., .FALSE. /)

!      real(r8), pointer :: ACG_bnd(:,:,:)

      integer :: LBi, UBi, LBj, UBj
      integer :: Imin, Imax, Jmin, Jmax
      integer :: i, j, d, d_bnd
      integer :: ND_BND

# ifdef AC_OBC
      integer :: ILB, IUB, JLB, JUB
# endif

# ifdef AC_OBC
!
!  Lower and upper bounds for nontiled boundary arrays.
!
      ILB=LOWER_BOUND_I
      IUB=UPPER_BOUND_I
      JLB=LOWER_BOUND_J
      JUB=UPPER_BOUND_J
!
# endif

!
!  Get the number of wave directional bins for the boundary.
!

      ND_BND=WAVEB(ng)%ND_BND

# ifdef PROFILE
!
!-----------------------------------------------------------------------
!  Turn on input data time wall clock.
!-----------------------------------------------------------------------
!
      CALL wclock_on (ng, iNLM, 3)
# endif

# ifdef WEST_AC_OBC
      CALL get_ngfld (ng, iNLM, idACbc(iwest), ncIWBRYid(ng), 1,        &
     &                IWBNDname(ng), update(1),                         &
     &                JLB, JUB, ND_BND, 2, 0, Mm(ng)+1, ND_BND,         &
     &                WAVEB(ng)%ACG_west(JLB,1,1))
# endif
# ifdef EAST_AC_OBC
      CALL get_ngfld (ng, iNLM, idACbc(ieast), ncIWBRYid(ng), 1,        &
     &                IWBNDname(ng), update(1),                         &
     &                JLB, JUB, ND_BND, 2, 0, Mm(ng)+1, ND_BND,         &
     &                WAVEB(ng)%ACG_east(JLB,1,1))
# endif
# ifdef NORTH_AC_OBC
      CALL get_ngfld (ng, iNLM, idACbc(inorth), ncIWBRYid(ng), 1,       &
     &                IWBNDname(ng), update(1),                         &
     &                ILB, IUB, ND_BND, 2, 0, Lm(ng)+1, ND_BND,         &
     &                WAVEB(ng)%ACG_north(ILB,1,1))
# endif
# ifdef SOUTH_AC_OBC
      CALL get_ngfld (ng, iNLM, idACbc(isouth), ncIWBRYid(ng), 1,       &
     &                IWBNDname(ng), update(1),                         &
     &                ILB, IUB, ND_BND, 2, 0, Lm(ng)+1, ND_BND,         &
     &                WAVEB(ng)%ACG_south(ILB,1,1))
# endif

# ifdef PROFILE
!
!-----------------------------------------------------------------------
!  Turn off input data time wall clock.
!-----------------------------------------------------------------------
!
      CALL wclock_off (ng, iNLM, 3)
# endif
#endif
      RETURN
      END SUBROUTINE get_inwave_data
