#!/bin/bash

# Author:  Panagiotis Velissariou <pvelissariou@fsu.edu>
#                                 <velissariou.1@osu.edu>
# Version: 1.3
#
# Version - 1.3 Thu Jul 30 2015
# Version - 1.2 Sun Jul 28 2013
# Version - 1.1 Wed Feb 27 2013
# Version - 1.0 Wed Jul 25 2012

Make_MODELEnv()
{
  local file1 file2
  local pat_str="USE_MPI"

  local i model
  local mods use_mods val_mods

  file1="${1}"
  file2="${2}"

  getOceanModels

      mods=( ${ocnMODELS} )
  use_mods=( ${ocnUSEMODS} )
  val_mods=( ${ocnUSEVALS} )
  unset ocnMODELS ocnUSEMODS ocnUSEVALS
 
  if $( checkFILE -r ${file1} ); then
    echo "Creating -> ${file2}"
    install -m 0644 "${file1}" "${file2}"

    sed -i "s/\(^[ \t]*\)COUPLED_SYSTEM=\(.*\)/\1COUPLED_SYSTEM=${COUPLED_SYSTEM:-no}/g"    "${file2}"
    sed -i "s/\(^[ \t]*\)USE_MPI=\(.*\)/\1USE_MPI=${USE_MPI:-no}/g"                         "${file2}"
    sed -i "s/\(^[ \t]*\)USE_OpenMP=\(.*\)/\1USE_OpenMP=${USE_OpenMP:-no}/g"                "${file2}"
    sed -i "s/\(^[ \t]*\)USE_ROMS=\(.*\)/\1USE_ROMS=${USE_ROMS:-no}/g"                      "${file2}"
    sed -i "s/\(^[ \t]*\)USE_ADJOINT=\(.*\)/\1USE_ADJOINT=${USE_ADJOINT:-no}/g"             "${file2}"
    sed -i "s/\(^[ \t]*\)USE_TANGENT=\(.*\)/\1USE_TANGENT=${USE_TANGENT:-no}/g"             "${file2}"
    sed -i "s/\(^[ \t]*\)USE_REPRESENTER=\(.*\)/\1USE_REPRESENTER=${USE_REPRESENTER:-no}/g" "${file2}"
    sed -i "s/\(^[ \t]*\)USE_HYCOM=\(.*\)/\1USE_HYCOM=${USE_HYCOM:-no}/g"                   "${file2}"
    # Append USE_MODEL for all other supported models.
    for ((i = 0; i < ${#mods[@]}; i++))
    do
      model="${mods[${i}]}"
      if [ "X${model}" != "XROMS" -a "X${model}" != "XHYCOM" ]; then
        sed -i "/\(^[[:space:]]*\)${pat_str}=\(.*\)/a ${use_mods[${i}]}=${val_mods[${i}]}" "${file2}"
        pat_str="${use_mods[${i}]}"
      fi
    done
    sed -i "s/\(^[ \t]*\)USE_WRF=\(.*\)/\1USE_WRF=${USE_WRF:-no}/g"                         "${file2}"
    sed -i "s/\(^[ \t]*\)USE_SWAN=\(.*\)/\1USE_SWAN=${USE_SWAN:-no}/g"                      "${file2}"
    sed -i "s/\(^[ \t]*\)USE_SED=\(.*\)/\1USE_SED=${USE_SED:-no}/g"                         "${file2}"
    sed -i "s/\(^[ \t]*\)USE_ICE=\(.*\)/\1USE_ICE=${USE_ICE:-no}/g"                         "${file2}"
  fi
}

Make_RUNScripts()
{
  local file1 file2

  file1="${1}"
  file2="${2}"

  if $( checkFILE -r ${file1} ); then
    echo "Creating -> ${file2}"
    install -m 0755 "${file1}" "${file2}"

    sed -i "s/\(^[ \t]*\)ocnDOMS=\(.*\)/\1ocnDOMS=${NestedGrids:-1}/g"   "${file2}"
    sed -i "s/\(^[ \t]*\)swanDOMS=\(.*\)/\1swanDOMS=${NestedGrids:-1}/g" "${file2}"
    sed -i "s/_MY_BINDIR_/./g"                                           "${file2}"
    sed -i "s/_MY_MODEL_EXE_/${model_name:+-${model_name}}/g"            "${file2}"
    sed -i "s/_MY_PROG_EXE_/${model_exe}/g"                              "${file2}"
    sed -i "s/_MY_CASE_ID_/${CASEID:-}/g"                                "${file2}"
    sed -i "s/_MY_VER_STR_/${VER_STR:-}/g"                               "${file2}"
    sed -i "s/_MY_MODFILES_/${MODFILES:+${MODFILES}}/g"                  "${file2}"
    sed -i "s/_MY_COMPSYS_/${COMPSYS:+${COMPSYS}}/g"                     "${file2}"
    sed -i "s/_MY_MPISYS_/${MPISYS:+-${MPISYS}}/g"                       "${file2}"
    sed -i "s/_MY_MPIVER_/${MPISYS:+-${MPISYS}${MPIVER:+-${MPIVER}}}/g"  "${file2}"
  fi
}

#============================================================
# BEG:: SCRIPT INITIALIZATION
#============================================================
[[ ! :$PATH: == *:".":* ]] && export PATH="${PATH}:."

##########
# Script identification variables
# The script name and the directory where this script is located
scrNAME="$( basename ${0} )"
scrDIR="$( dirname ${0} )"
pushd ${scrDIR} > /dev/null 2>&1
scrDIR="$( pwd )"
popd > /dev/null 2>&1

##########
# Set/Check the "rootDIR" variable
# This is the directory of the GOMSYS source code
rootDIR=${rootDIR:=${scrDIR}}

if [ ! -d "${rootDIR}" ]; then
  echo " ### ERROR: in ${scrNAME}"
  echo "       The supplied value for:"
  echo "       rootDIR = ${rootDIR}"
  echo "       is not a valid directory. This variable is essential"
  echo "       for this script to be executed properly."
  echo "     Exiting now ..."
  echo
  exit 1
fi

##########
# Source the Utility Functions so they are available
# to this script
if [ -f functions_build ]; then
  source functions_build
else
  echo " ### ERROR: in ${scrNAME}"
  echo "       Couldn't locate the file <functions_build> that contains"
  echo "       all the necessary utility functions required for this"
  echo "       script to be executed properly."
  echo "     Exiting now ..."
  exit 1
fi

COLORSET="$( getYesNo "${COLORSET:-yes}" )"
  [ "${COLORSET:-no}" = "no" ] && unset COLORSET

#============================================================
# END:: SCRIPT INITIALIZATION
#============================================================


#============================================================
# BEG:: SETTING DEFAULTS AND/OR THE USER INPUT
#============================================================
OCEAN_MODEL_NAME=${OCEAN_MODEL_NAME:-}
MODNAMEBASE=gomfs

Get_LibName

##########
# First, source the file pointed by the GOMSYS_ENV environment variable
# (if it is set) to get any user defined values. If GOMSYS_ENV is
# not set try a file called "gomsys_env.sh" next. We source
# these files before calling ParseArgsBuild below in case the user has
# already set this environment.
unset envISSET
if [ -z "${GOMSYS_ENV:-}" ]; then
  srchPATH="./ ${scrDIR} ${rootDIR}
            ${scrDIR}/.. ${scrDIR}/../scripts
            ${rootDIR}/scripts"
  for spath in ${srchPATH}
  do
    if $( checkFILE -r ${spath}/gomsys_env.sh ); then
      source ${spath}/gomsys_env.sh
      export envISSET=1
      break
    fi
  done
  unset spath srchPATH
else
  if $( checkFILE -r "${GOMSYS_ENV}" ); then
    source "${GOMSYS_ENV}"
    export envISSET=2
  fi
fi

export WRF_OS="${WRF_OS:-$( uname )}"
export WRF_MACH="${WRF_MACH:-$( uname -m )}"
export WRF_EM_CORE="${WRF_EM_CORE:-1}"
export WRF_NMM_CORE="${WRF_NMM_CORE:-0}"

#########
# Call ParseArgsBuild to get any additional user input
# (if requested).
ParseArgsBuild "${@}"

#============================================================
# END:: SETTING DEFAULTS AND/OR THE USER INPUT
#============================================================


#============================================================
# BEG:: CHECK THE VARIABLES
# Check the environment variables and adjust as needed
# User defined environmental variables. See the GOMSYS makefile for
# details on other options the user might want to set here. Be sure to
# leave the switches meant to be off set to an empty string or commented
# out. Any string value (including off) will evaluate to TRUE in
# conditional if-statements.
#============================================================

# ------------------------------------------------------------
# GOMSYS_APPLICATION should be set in any case.
if [ -z "${GOMSYS_APPLICATION:-}" ]; then
  procError "The GOMSYS_APPLICATION variable is not set:" \
            "  GOMSYS_APPLICATION = ${GOMSYS_APPLICATION:-UNDEF}" \
            "Use: ${scrNAME} -h, to see all available options"
fi

# ------------------------------------------------------------
# The module file(s) to load (if any).
LoadEnvModules ${MODFILES:-}


# ------------------------------------------------------------
# The Fortran compiler to use.
COMPILER=${COMPILER:-ifort}
Get_Compiler "${COMPILER}"

# ------------------------------------------------------------
# Check for the important modelling system directories.
export MY_ROOT_DIR="${MY_ROOT_DIR:-${rootDIR}}"
export MY_PROJECT_DIR="${MY_PROJECT_DIR:-}"
export MY_GOMSYS_SRC="${MY_GOMSYS_SRC:-${MY_ROOT_DIR}}"
export OCN_DIR=$( GetOcnDir )
export WRF_DIR="${WRF_DIR:-${MY_GOMSYS_SRC:+${MY_GOMSYS_SRC}/}WRF}"
export WPS_DIR="${WPS_DIR:-${MY_GOMSYS_SRC:+${MY_GOMSYS_SRC}/}WPS}"
export SWAN_DIR="${SWAN_DIR:-${MY_GOMSYS_SRC:+${MY_GOMSYS_SRC}/}SWAN}"
export HYCOM_ALL="${HYCOM_ALL:-${MY_GOMSYS_SRC:+${MY_GOMSYS_SRC}/}ALL}"

Check_SystemDirs

# Get all the models that are active in the coupled system
# that is, OCEAN/WRF/SWAN and the coupling status (yes/no) of the system
SystemActiveModels

# ------------------------------------------------------------
# Get the versions of the modeling components.
# Get the ROMS Version, Revision and Release Date
MY_MOD_DIR="${MY_GOMSYS_SRC:+${MY_GOMSYS_SRC}/}ROMS"
if $( checkDIR -rx "${MY_MOD_DIR}" ); then
  if $( checkFILE -r ${MY_MOD_DIR}/Modules/mod_ncparam.F ); then
    ROMS_VER="$( cat ${MY_MOD_DIR}/Modules/mod_ncparam.F | grep -Ei 'version.*=' )"
    ROMS_VER="$( echo "${ROMS_VER}" | sed -e 's/.*[vV][eE][rR][sS][iI][oO][nN].*=//g' )"
    ROMS_VER="$( echo "${ROMS_VER}" | sed -e 's/'\''//g' | awk '{print $1}' | sed -e 's/[vV]//g' )"
  fi
  if $( checkFILE -r ${MY_MOD_DIR}/Version ); then
    ROMS_REV="$( cat ${MY_MOD_DIR}/Version | grep -Ei revision: )"
    ROMS_REV="$( echo "${ROMS_REV}" | sed -e 's/.*[rR][eE][vV][iI][sS][iI][oO][nN]//g' )"
    ROMS_REV="$( echo "${ROMS_REV}" | sed -e 's/[=;:,_()\{\}\$\\]/ /g' | awk '{print $1}' )"
    ROMS_DATE="$( cat ${MY_MOD_DIR}/Version | grep -Ei changeddate: | sed -e 's/.*[aA][tT][eE]://g' )"
    ROMS_DATE="$( echo "${ROMS_DATE}" | sed -e 's/\$//g' | awk '{print $1}' )"
    ROMS_DATE="$( getDate --date="${ROMS_DATE}" --fmt='+%m-%d-%Y' )"
  fi
fi
unset MY_MOD_DIR

# Get the HYCOM Version, Revision and Release Date
MY_MOD_DIR="${MY_GOMSYS_SRC:+${MY_GOMSYS_SRC}/}HYCOM"
if $( checkDIR -rx "${MY_MOD_DIR}" ); then
  HYCOM_VER=2.2.77i
  #HYCOM_VER=2.2.98ZA

  HYCOM_SRC="${MY_MOD_DIR}/src_${HYCOM_VER}"
  if $( checkDIR -rx "${HYCOM_SRC}" ); then
    case "${HYCOM_VER}" in
      "2.2.77i")
        HYCOM_REV=
        HYCOM_DATE="$( getDate --date="2012-01-11" --fmt='+%m-%d-%Y' )"
        ;;
      "2.2.98ZA")
        HYCOM_REV=
        HYCOM_DATE="$( getDate --date="2015-03-04" --fmt='+%m-%d-%Y' )"
        ;;
      *)
        unset HYCOM_VER HYCOM_REV HYCOM_DATE
        ;;
    esac
  fi
fi
unset HYCOM_SRC MY_MOD_DIR

# Get the WRF Version
if $( checkDIR -rx "${WRF_DIR}" ); then
  if $( checkFILE -r ${WRF_DIR}/inc/version_decl ); then
    WRF_VER="$( cat ${WRF_DIR}/inc/version_decl | grep -Ei release_version )"
    WRF_VER="$( echo "${WRF_VER}" | sed -e 's/.*[vV][eE][rR][sS][iI][oO][nN].*=//g' )"
    WRF_VER="$( echo "${WRF_VER}" | sed -e 's/'\''//g' | awk '{print $1}' | sed -e 's/[vV]//g' )"
  fi
  if $( checkFILE -r ${WRF_DIR}/README ); then
    if [ -n "${WRF_VER:+1}" ]; then
      WRF_DATE="$( cat ${WRF_DIR}/README | grep -Ei "^WRF.*Version.*${WRF_VER}.*" | sed -e "s/.*${WRF_VER}//g" )"
      WRF_DATE="$( echo ${WRF_DATE} | sed -e 's/[=;,_()\{\}\\]/ /g' )"
      WRF_REV="$( cat ${WRF_DIR}/README | grep -Ei "Version.*${WRF_VER}.*released on " | sed -e 's/.*[rR][eE][vV]//g' )"
      WRF_REV="$( echo "${WRF_REV}" | sed -e 's/[=;,_()\{\}\\]/ /g' | awk '{print $1}' )"
    fi
    WRF_DATE="$( getDate --date="${WRF_DATE}" --fmt='+%m-%d-%Y' )"
  fi
fi

# Get the WPS Version
if $( checkDIR -rx "${WPS_DIR}" ); then
  if $( checkFILE -r ${WPS_DIR}/README ); then
    WPS_STR=
    WPS_STR="$( cat ${WPS_DIR}/README | grep -Ei "Pre-Processing System Version" )"
    WPS_STR="$( echo ${WPS_STR} | sed -e 's/[=;,_()\{\}\\]/ /g' )"
    WPS_VER="$( echo ${WPS_STR} | sed -e 's/.*[vV][eE][rR][sS][iI][oO][nN]//g' | awk '{print $1}' )"
    if [ -n "${WPS_VER:+1}" ]; then
      WPS_DATE="$( echo ${WPS_STR} | sed -e "s/.*[vV][eE][rR][sS][iI][oO][nN].*${WPS_VER}//g" )"
      WPS_DATE="$( echo ${WPS_DATE} | sed -e "s/[rR][eE][vV].*//g" )"
      WPS_REV="$( echo ${WPS_STR} | sed -e "s/.*[rR][eE][vV][iI][sS][iI][oO][nN]//g" | awk '{print $1}' )"
      unset WPS_STR
    fi
    WPS_DATE="$( getDate --date="${WPS_DATE}" --fmt='+%m-%d-%Y' )"
    unset WPS_STR
  fi
fi

# Get the SWAN Version
if $( checkDIR -rx "${SWAN_DIR}" ); then
  if $( checkFILE -r ${SWAN_DIR}/Src/swanmain.F ); then
    SWAN_VER="$( cat ${SWAN_DIR}/Src/swanmain.F | grep -Ei ".*VERNUM.*=" )"
    SWAN_VER="$( echo "${SWAN_VER}" | sed -e 's/.*[vV][eE][rR][nN][uU][mM].*=//g' | awk '{print $1}' )"
  fi
  if $( checkFILE -r ${SWAN_DIR}/Src/swanuse.tex ); then
    if [ -n "${SWAN_VER:+1}" ]; then
      SWAN_DATE="$( cat ${SWAN_DIR}/Src/swanuse.tex | grep -Ei "\(Version.*${SWAN_VER}" | sort -u )"
      SWAN_DATE="$( echo ${SWAN_DATE} | sed -e 's/.*([vV][eE][rR][sS][iI][oO][nN].*,//g' | sed -e 's/[(),.]//g' )"
      SWAN_DATE="$( echo ${SWAN_DATE} | awk '{print $1 " 1, " $2}' )"
    fi
    SWAN_DATE="$( getDate --date="${SWAN_DATE}" --fmt='+%m-%d-%Y' )"
  fi
fi

# ------------------------------------------------------------
# Check for an MPI/OpenMP setup
if [ "${USE_MPI:-no}" = "yes" ] && [ "${USE_OpenMP:-no}" = "yes" ]; then
  procError "   USE_MPI = ${USE_MPI}" \
            "USE_OpenMP = ${USE_OpenMP}" \
            "Only one of the USE_MPI/USE_OpenMP variables can be set" \
            "Please adjust your command line or the environment file accordingly"
fi

if [ "${USE_MPI:-no}" = "no" ]; then
  unset USE_MPI
  unset USE_MPIF90
  unset USE_PNETCDF
  unset USE_PARALLEL_IO
else
  # force the use of mpif90 when USE_MPI="yes"
  USE_MPIF90="yes"
  Get_MpiCompiler "mpif90"
  if [ "${USE_PARALLEL_IO:-no}" = "yes" ]; then
    USE_PNETCDF=yes
    USE_NETCDF4=yes
    unset USE_NETCDF3
  else
    unset USE_PNETCDF
  fi
fi

[ "${USE_NETCDF4:-no}" = "yes" -o "${USE_PNETCDF:-no}" = "yes" -o "${USE_PARALLEL_IO:-no}" = "yes" ] && \
  unset USE_NETCDF3

# ------------------------------------------------------------
# Get the path of the NetCDF headers and libraries
Get_NetCDFPath
if [ "$?" -ne 0 ]; then
  procError "No suitable NetCDF header/libraries found in the system" \
            "User/Script variables used:" \
            "    USE_NETCDF3 = ${USE_NETCDF3:-no}" \
            "    USE_NETCDF4 = ${USE_NETCDF4:-no}" \
            " NETCDF_VERSION = ${NETCDF_VERSION:-UNDEF}" \
            "    NETCDF_ROOT = ${NETCDF_ROOT:-UNDEF}" \
            "      NC_CONFIG = ${NC_CONFIG:-UNDEF}" \
            "  NETCDF_INCDIR = ${NETCDF_INCDIR:-UNDEF}" \
            "  NETCDF_LIBDIR = ${NETCDF_LIBDIR:-UNDEF}" \
            "NETCDF_PARALLEL = ${NETCDF_PARALLEL:-no}"
fi

# ------------------------------------------------------------
# Get the path of the NetCDF headers and libraries
Get_MCTPath
if [ "$?" -ne 0 ]; then
  procError "No suitable MCT header/libraries found in the system" \
            "User/Script variables used:" \
            "     USE_MCT = ${USE_MCT:-no}" \
            " MCT_VERSION = ${MCT_VERSION:-UNDEF}" \
            "    MCT_ROOT = ${MCT_ROOT:-UNDEF}" \
            "  MCT_INCDIR = ${MCT_INCDIR:-UNDEF}" \
            "  MCT_LIBDIR = ${MCT_LIBDIR:-UNDEF}" \
            "MCT_PARALLEL = ${MCT_PARALLEL:-no}"
fi

# ------------------------------------------------------------
# Get the path of the HDF5 headers and libraries (possibly for parallel IO)
Get_HDF5Path
if [ "$?" -ne 0 ]; then
  procError "No suitable HDF5 header/libraries found in the system" \
            "User/Script variables used:" \
            "    NETCDF_ROOT = ${NETCDF_ROOT:-UNDEF}" \
            "      NC_CONFIG = ${NC_CONFIG:-UNDEF}" \
            "  NETCDF_INCDIR = ${NETCDF_INCDIR:-UNDEF}" \
            "  NETCDF_LIBDIR = ${NETCDF_LIBDIR:-UNDEF}" \
            " NETCDF_VERSION = ${NETCDF_VERSION:-UNDEF}" \
            "NETCDF_PARALLEL = ${NETCDF_PARALLEL:-no}" \
            "USE_PARALLEL_IO = ${USE_PARALLEL_IO:-no}" \
            "    USE_PNETCDF = ${USE_PNETCDF:-no}" \
            "       USE_HDF5 = ${USE_HDF5:-no}" \
            "      HDF5_ROOT = ${NETCDF_ROOT:-UNDEF}" \
            "    HDF5_INCDIR = ${HDF5_INCDIR:-UNDEF}" \
            "    HDF5_LIBDIR = ${HDF5_LIBDIR:-UNDEF}" \
            "   HDF5_VERSION = ${HDF5_VERSION:-UNDEF}" \
            "  HDF5_PARALLEL = ${HDF5_PARALLEL:-no}"
fi

# ------------------------------------------------------------
# Get the path of the PNetCDF headers and libraries
Get_PNetCDFPath
if [ "$?" -ne 0 ]; then
  procError "No suitable PNetCDF header/libraries found in the system" \
            "User/Script variables used:" \
            "    NETCDF_ROOT = ${NETCDF_ROOT:-UNDEF}" \
            "      NC_CONFIG = ${NC_CONFIG:-UNDEF}" \
            "  NETCDF_INCDIR = ${NETCDF_INCDIR:-UNDEF}" \
            "  NETCDF_LIBDIR = ${NETCDF_LIBDIR:-UNDEF}" \
            " NETCDF_VERSION = ${NETCDF_VERSION:-UNDEF}" \
            "NETCDF_PARALLEL = ${NETCDF_PARALLEL:-no}" \
            "USE_PARALLEL_IO = ${USE_PARALLEL_IO:-no}" \
            "    USE_PNETCDF = ${USE_PNETCDF:-no}" \
            "PNETCDF_VERSION = ${PNETCDF_VERSION:-UNDEF}" \
            "   PNETCDF_ROOT = ${NETCDF_ROOT:-UNDEF}" \
            " PNETCDF_INCDIR = ${PNETCDF_INCDIR:-UNDEF}" \
            " PNETCDF_LIBDIR = ${PNETCDF_LIBDIR:-UNDEF}"
fi

# ------------------------------------------------------------
# Get the path of the Jasper headers and libraries
# if requested.

if [ "${USE_JASPER:-no}" = "yes" ]; then
  theFiles="jasper.h jas_version.h"
  Check_Includes "${JASPER_INCDIR}" "${theFiles}" warning
  if [ $? -ne 0 ]; then
    unset JASPER_INCDIR
    procError "The Jasper headers not found"
  fi

  theFiles="libjasper.*"
  Check_Libraries "${JASPER_LIBDIR}" "${theFiles}" warning
  if [ $? -ne 0 ]; then
    unset JASPER_LIBDIR
    procError "The Jasper libraries not found"
  fi
else
  unset JASPER_ROOT JASPER_INCDIR JASPER_LIBDIR
fi

# ------------------------------------------------------------
# Get the path of the NCARG headers and libraries
# if requested.

if [ "${USE_NCL:-no}" = "yes" ]; then
  theFiles="libncarg.* libncarg_gks.* libncarg_c.*"
  Check_Libraries "${NCL_LIBDIR}" "${theFiles}" warning
  if [ $? -ne 0 ]; then
    unset NCL_LIBDIR
    procError "The NCAR libraries not found"
  fi
else
  unset NCL_ROOT NCL_INCDIR NCL_LIBDIR
fi

# ------------------------------------------------------------
# Make sure that all variables are set correctly
SystemDefaults

Adjust_YESNOVars

#============================================================
# END:: CHECK THE VARIABLES
#============================================================


#============================================================
# BEG:: EXPORT ALL USER DEFINED VARIABLES
#============================================================

export CLEAN
export CLEANWRF
export CLEANWPS
export PARMAKE_NCPUS="${PARMAKE_NCPUS:+-j ${PARMAKE_NCPUS}}"
export MODFILES
export USE_DEBUG
export COMPILER
export CASEID
export USE_MPI
export USE_MPIF90
export USE_OpenMP
export COUPLED_SYSTEM

export MY_ROOT_DIR
export MY_PROJECT_DIR
export OCN_DIR
export GOMSYS_APPLICATION
export MY_GOMSYS_SRC

export MY_HEADER_DIR
export MY_ANALYTICAL_DIR
export SCRATCH_DIR
export BINDIR
export COMPILERS
export MY_CPP_FLAGS
export NestedGrids

export USE_SED
export USE_ICE

export USE_ROMS
export USE_ADJOINT
export USE_TANGENT
export USE_REPRESENTER
export ROMS_DIR
export ROMS_VER
export ROMS_REV
export ROMS_DATE

export USE_HYCOM
export BUILD_ALL
export HYCOM_ALL
export HYCOM_DIR
export HYCOM_VER
export HYCOM_REV
export HYCOM_DATE

export USE_WRF
export WRF_OS
export WRF_MACH
export WRF_EM_CORE
export WRF_NMM_CORE
export WRFIO_NCD_LARGE_FILE_SUPPORT
export WRF_DIR
export WRF_VER
export WRF_REV
export WRF_DATE

export WPS_DIR
export WPS_VER
export WPS_REV
export WPS_DATE

export USE_SWAN
export USE_INWAVE
export USE_REFDIF
export SWAN_DIR
export SWAN_VER
export SWAN_REV
export SWAN_DATE

export USE_NETCDF3
export USE_NETCDF4
export USE_LARGE
export NETCDF_ROOT
export NC_CONFIG
export NETCDF_INCDIR
export NETCDF_LIBDIR
export NETCDF_VERSION
export NETCDF_MAJOR
export NETCDF_MINOR
export NETCDF_BUILD
export NETCDF_PARALLEL

export USE_HDF5
export HDF5_ROOT
export HDF5_INCDIR
export HDF5_LIBDIR
export HDF5_VERSION
export HDF5_MAJOR
export HDF5_MINOR
export HDF5_BUILD
export HDF5_PARALLEL
export USE_PARALLEL_IO

export USE_PNETCDF
export PNETCDF_INCDIR
export PNETCDF_LIBDIR
export PNETCDF_VERSION
export PNETCDF_MAJOR
export PNETCDF_MINOR
export PNETCDF_BUILD

export USE_ARPACK
export ARPACK_ROOT
export ARPACK_INCDIR
export ARPACK_LIBDIR

export USE_PARPACK
export PARPACK_ROOT
export PARPACK_INCDIR
export PARPACK_LIBDIR

export USE_JASPER
export JASPER_ROOT
export JASPER_INCDIR
export JASPER_LIBDIR

export USE_NCL
export NCL_ROOT
export NCL_INCDIR
export NCL_LIBDIR
export NCL_XLIBS

export USE_MCT
export MCT_ROOT
export MCT_INCDIR
export MCT_LIBDIR
export MCT_PARALLEL
export MPEU_INCDIR="${MCT_INCDIR:-}"
export MPEU_LIBDIR="${MCT_LIBDIR:-}"

export USE_ESMF
export ESMF_ROOT
export ESMF_INCDIR
export ESMF_LIBDIR
  
export FORT="${COMPILER:-}"
export COMPSYS
export MPISYS
export MPIVER

#============================================================
# END:: EXPORT ALL USER DEFINED VARIABLES
#============================================================


#============================================================
# BEG:: ECHO THE FINAL VALUES OF THE ASSIGNED PARAMETERS
#       AND THE USER'S RESPONSE
#============================================================

Print_Vars
echo

checkOceanModels

echo -n "Are these values correct? [y/n]: "
echo_response=
while [ -z "${echo_response}" ] ; do
  read echo_response
  echo_response="$( getYesNo "${echo_response}" )"
done

if [ "${echo_response:-no}" = "no" ]; then
  echo
  echo "User responded: ${echo_response}"
  echo "Exiting now ..."
  echo
  exit 1
fi

unset echo_response

#============================================================
# END:: ECHO THE FINAL VALUES OF THE ASSIGNED PARAMETERS
#       AND THE USER'S RESPONSE
#============================================================


#============================================================
# BEG:: START THE COMPILATION
#============================================================

ulimit -s unlimited

# ------------------------------------------------------------
# These are for forcing the model to use the *_INCDIR and *_LIBDIR variables
unset USE_NETCDF3
unset USE_NETCDF4
unset NETCDF
unset USE_PNETCDF
unset PNETCDF

VER_STR=
if [ "${VERSIONING:-0}" -gt 0 ]; then
  VER_STR="${USE_PARALLEL_IO:+-pio}"
  VER_STR="${VER_STR}${COMPSYS:+-${COMPSYS}}${MPISYS:+-${MPISYS}${MPIVER:+-${MPIVER}}}"
  VER_STR="${VER_STR#-}"
fi
EXE_STR="${CASEID:-}${VER_STR:+-${VER_STR}}"
EXE_STR="${EXE_STR#-}"
export VER_STR EXE_STR

# ------------------------------------------------------------
# Create the final name of the resulting executable
if [ -n "${USE_MPI:+1}" ]; then
  mod_sfx=M
elif [ -n "${USE_OpenMP:+1}" ]; then
  mod_sfx=O
else
  mod_sfx=S
fi
[ -n "${USE_DEBUG:+1}" ] && mod_sfx=G

mod_id=
#[ -n "${USE_WRF:+1}"  ] && mod_id=${mod_id}W
#[ -n "${USE_SWAN:+1}" ] && mod_id=${mod_id}S
#[ -n "${USE_SED:+1}"  ] && mod_id=${mod_id}D
#[ -n "${USE_ICE:+1}"  ] && mod_id=${mod_id}I

if [ -n "${USE_WRF:+1}" -a -z "${USE_HYCOM}" -a -z "${USE_SWAN}" ]; then
  model_name=wrf
  model_exe=wrf${EXE_STR:+-${EXE_STR}}.exe
else
  model_name=${MODNAMEBASE}${mod_sfx}
  model_exe=${MODNAMEBASE}${mod_sfx}${mod_id:+-${mod_id}}${EXE_STR:+-${EXE_STR}}
fi


########## BEG:: DO_COMPILE
if [ "${DO_COMPILE}" -gt 0 ]; then
  # Compile (the binaries will go to BINDIR set above).
  # (BINDIR is internally set to be: BINDIR=MY_GOMSYS_SRC).
  pushd ${MY_GOMSYS_SRC} > /dev/null 2>&1

    # Make link HYCOM/src to real HYCOM version used
    if [ -n "${USE_HYCOM:+1}" ]; then
      my_file="HYCOM/src_${HYCOM_VER}/hycom.F"
      if $( checkFILE -r "${my_file}" ); then
        [ -f "Master/hycom.F" ] && rm -f Master/hycom.F
        cp -f ${my_file} Master/
      else
        procError "The source file \"${my_file}\" does not exist or," \
                  "permissions are not valid for the current user." \
                  "This file is essential for a successful compilation of HYCOM."
      fi

      linkFILE HYCOM/src_${HYCOM_VER} HYCOM/src
    fi

    # Modify the makefile-tmpl and create the makefile
    INP_MAKE=makefile-tmpl
    if $( checkFILE -r ${INP_MAKE} ); then
      [ -f makefile ] && rm -f makefile
      cp -f ${INP_MAKE} makefile
      for imod in S G M O wrf.exe
      do
        if [ "${imod}" = "wrf.exe" ]; then
          imod_n="${imod}"
          imod_p="$(echo ${imod_n%%.exe})"
          imod_s="$(echo "${imod_n}" | sed "s/${imod_p}//g")"
          perl -pi -e  "s@\\$\(BINDIR\)/${imod_n}@\\$\(BINDIR\)/${imod_p}${EXE_STR:+-${EXE_STR}}${imod_s}@g" \
            makefile
        else
          imod_n="${MODNAMEBASE}${imod}"
          perl -pi -e  "s@\\$\(BINDIR\)/_name_${imod}@\\$\(BINDIR\)/${imod_n}${EXE_STR:+-${EXE_STR}}@g" \
            makefile
        fi
      done
    else
      procError "The template makefile: ${INP_MAKE}" \
                "is not found. This file is required for" \
                "the compilation of the modeling components."
    fi

    # Make a backup copy of the GOMSYS binary.
    [ -f "${BINDIR}/${model_exe}" ] && \
      mv -f ${BINDIR}/${model_exe} ${BINDIR}/${model_exe}.backup

    # Clean in the GOMSYS directory.
    Clean_WRF
    for imod in ${SUPPORTED_OCEAN_MODELS}
    do
        command -v Clean_${imod} >/dev/null 2>&1 && \
      Clean_${imod}
    done

    # Configure GOMSYS/WRF/WPS (only if USE_WRF or BUILD_WPS are set).
    Configure_WRF
    [ $? -ne 0 ] && \
      procError "WRF configuration failed"

    if [ -n "${USE_WRF:+1}" ]; then
      # Compile WRF.
      make ${PARMAKE_NCPUS} wrf
      [ $? -ne 0 ] && \
        procError "WRF compilation failed"

      # Compile in the WPS directory. This has to be done after WRF
      # has been compiled successfully.
      if [ -n "${BUILD_WPS:+1}" ]; then
        pushd ${WPS_DIR} > /dev/null 2>&1

          # Compile WPS
          ./compile wps
          [ $? -ne 0 ] && \
            procError "WPS compilation failed"

          # Compile WPS utilities
          ./compile util
          [ $? -ne 0 ] && \
            procError "WPS utilities compilation failed"
        popd > /dev/null 2>&1
      fi
    fi

    # Compile the Ocean Model
    make ${PARMAKE_NCPUS}
      [ $? -ne 0 ] && \
        procError "OCEAN/WAVE compilation failed"

    # Remove the modified makefile
    #[ -f makefile ] && rm -f makefile
  popd > /dev/null 2>&1
fi

if [ -n "${USE_HYCOM:+1}" ]; then
  # Compile in the HYCOM ALL directory.
  # This compilation is independent of HYCOM and can be done
  # outside the compilation of the modeling components.
  # Binaries are installed in HYCOM_ALL/Build.
  # This needs to be done just once.
  if [ -n "${BUILD_ALL:+1}" ]; then
    Clean_HYCOM_ALL
    pushd ${HYCOM_ALL} > /dev/null 2>&1
      make install
    popd > /dev/null 2>&1
  fi
fi
########## END:: DO_COMPILE


########## BEG:: INSTALL BINARIES AND DATA
if [ "${MY_PROJECT_DIR}" != "${MY_GOMSYS_SRC}" ]; then
  if [ "${BINDIR}" != "${MY_PROJECT_DIR}" ]; then
    install -m 0755 -p "${BINDIR}/${model_exe}" "${MY_PROJECT_DIR}/${model_exe}"
  fi

  # ------------------------------------------------------------
  # Install model binaries and data.
  getOceanModels

  for imod in ${ocnMODELS}
  do
    command -v Install_${imod}Files >/dev/null 2>&1 && \
      Install_${imod}Files
  done

  Install_WRFFiles
  Install_WPSFiles
  Install_SWANFiles
  Install_SEDFiles

  # ------------------------------------------------------------
  # Install the utility programs.
  # (A) GNU Parallel
  inp_dir="Utilities/parallel"
  if $( checkDIR -r "${inp_dir}" ); then
    my_files="parallel niceload sem sql"
    out_dir="${MY_PROJECT_DIR}/Bin"
    makeDIR "${out_dir}"

    for my_file in ${my_files}
    do
      my_file_inp="${inp_dir}/${my_file}"
      my_file_out="${out_dir}/${my_file}"

      if $( checkFILE -r "${my_file_inp}" ); then
        echo "Installing -> ${my_file_out}"
        install -m 0755 -p ${my_file_inp} ${my_file_out}
      else
        procWarn "Couldn't install the utility program:" \
                  "  program = ${my_file:-UNDEF}"
      fi
    done
  fi

  # ------------------------------------------------------------
  # Install helper files and data.
  my_files="functions_common functions_run"
  for my_file in ${my_files}
  do
    my_file_out="${MY_PROJECT_DIR}/${my_file}"

    if $( checkFILE -r "${my_file}" ); then
      if $( checkFILE -r "${my_file_out}" ); then
        my_file_out=${my_file_out}.new
      fi
      echo "Installing -> ${my_file_out}"
      install -m 0644 -p ${my_file} ${my_file_out}
    else
      procError "Couldn't install the required file:" \
                "  file = ${my_file:-UNDEF}"
    fi
  done

  if [ -n "${COUPLED_SYSTEM:+1}" ]; then
    my_file=sample-coupling.in-tmpl
    if $( checkFILE -r "${my_file}" ); then
      my_file="$( basename $( echo ${my_file} | sed -e 's/sample-//g' ) )"
      my_file_out="${MY_PROJECT_DIR}/${my_file}"
      if $( checkFILE -r "${my_file_out}" ); then
        my_file_out=${my_file_out}.new
      fi
      echo "Installing -> ${my_file_out}"
      install -m 0644 -p ${my_file} ${my_file_out}
    fi
  fi

  if [ -n "${USE_WRF:+1}" ]; then
    my_file=fort-namelist
    if $( checkFILE -r "${my_file}" ); then
      install -m 0755 -p ${my_file} ${MY_PROJECT_DIR}/${my_file}
    fi
  fi

  # ------------------------------------------------------------
  # Create and install the models_env file.
  my_file="models_env-tmpl"
  my_file_out="${MY_PROJECT_DIR}/models_env"
  if $( checkFILE -r "${my_file}" ); then
    if $( checkFILE -r "${my_file_out}" ); then
      my_file_out=${my_file_out}.new
    fi
    Make_MODELEnv "${my_file}" "${my_file_out}"
  else
    procWarn "Couldn't install the functions environment file:" \
              "  functions env = ${my_file:-UNDEF}"
  fi

  # ------------------------------------------------------------
  # Create and install the standalone mpirun script.
  my_file="script_mpirun-tmpl"
  my_file_out="${MY_PROJECT_DIR}/mpirun${model_name:+-${model_name}}.sh"
  if $( checkFILE -r "${my_file}" ); then
    if $( checkFILE -r "${my_file_out}" ); then
      my_file_out=${my_file_out}.new
    fi
    Make_RUNScripts "${my_file}" "${my_file_out}"
  else
    procWarn "Couldn't install the mpirun script:" \
              "  mpirun script = ${my_file:-UNDEF}"
  fi

  # ------------------------------------------------------------
  # Create the run-msub script.
  my_file="script_msub-tmpl"
  my_file_out="${MY_PROJECT_DIR}/msub${model_name:+-${model_name}}.sh"
  if $( checkFILE -r "${my_file}" ); then
    if $( checkFILE -r "${my_file_out}" ); then
      my_file_out=${my_file_out}.new
    fi
    Make_RUNScripts "${my_file}" "${my_file_out}"
  else
    procWarn "Couldn't install the msub script:" \
              "  msub script = ${my_file:-UNDEF}"
  fi

  # ------------------------------------------------------------
  # Create the run-slurm script.
  my_file="script_slurm-tmpl"
  my_file_out="${MY_PROJECT_DIR}/slurm${model_name:+-${model_name}}.sh"
  if $( checkFILE -r "${my_file}" ); then
    if $( checkFILE -r "${my_file_out}" ); then
      my_file_out=${my_file_out}.new
    fi
    Make_RUNScripts "${my_file}" "${my_file_out}"
  else
    procWarn "Couldn't install the slurm script:" \
              "  slurm script = ${my_file:-UNDEF}"
  fi
else
  procWarn "Please modify your variables MY_PROJECT_DIR/MY_GOMSYS_SRC." \
           "The destination directory MY_PROJECT_DIR points to the source directory:" \
           "  MY_PROJECT_DIR = ${MY_PROJECT_DIR:-UNDEF}" \
           "   MY_GOMSYS_SRC = ${MY_GOMSYS_SRC:-UNDEF}" \
           "Installation of binaries and data files is abandoned."
fi
########## END:: INSTALL BINARIES AND DATA
